import os
import subprocess

import pytest
import mock

from jobtriggers import utils
from jobtriggers import config
from jobtriggers import exceptions as exc


@pytest.mark.usefixtures('prepare')
class TestUtils():

    @pytest.fixture
    def prepare(self, monkeypatch):
        pass

    def test_check_yaml_scanning_error(self):
        file_data = '%'
        mocked_open = mock.mock_open(read_data=file_data)

        with mock.patch('__builtin__.open', mocked_open):
            with pytest.raises(exc.TriggerError):
                utils.parse_yaml_from_file('mocked_filename')

    def test_check_yaml_parsing_error(self):
        file_data = ':'
        mocked_open = mock.mock_open(read_data=file_data)

        with mock.patch('__builtin__.open', mocked_open):
            with pytest.raises(exc.TriggerError):
                utils.parse_yaml_from_file('mocked_filename')

    def test_check__should_schedule_true(self):
        file_data = '''
            scheduling:
                run: True

            input:
                args:
                    - koji_build
            '''
        mocked_open = mock.mock_open(read_data=file_data)

        with mock.patch('__builtin__.open', mocked_open):
            assert utils._should_schedule('mocked_filename', 'koji_build')

    def test_check__should_schedule_false(self):
        file_data = '''
            scheduling:
                run: False

            input:
                args:
                    - koji_build
            '''
        mocked_open = mock.mock_open(read_data=file_data)

        with mock.patch('__builtin__.open', mocked_open):
            assert not utils._should_schedule('mocked_filename', 'koji_build')

    def test_check__should_schedule_default(self):
        file_data = '''
            input:
                args:
                    - koji_build
            '''
        mocked_open = mock.mock_open(read_data=file_data)

        with mock.patch('__builtin__.open', mocked_open):
            assert utils._should_schedule('mocked_filename', 'koji_build')

    def test_check__should_schedule_false_item_type(self):
        file_data = '''
            input:
                args:
                    - koji_tag
            '''
        mocked_open = mock.mock_open(read_data=file_data)

        with mock.patch('__builtin__.open', mocked_open):
            assert not utils._should_schedule('mocked_filename', 'koji_build')

    def test_distgit_tasks(self, monkeypatch):
        mock_check_output = mock.Mock(return_value='')
        mock__should_schedule = mock.Mock(return_value=True)
        monkeypatch.setattr(subprocess, 'check_output', mock_check_output)
        monkeypatch.setattr(utils, '_should_schedule', mock__should_schedule)
        monkeypatch.setattr(os, 'listdir', lambda x: ['rpmlint'])
        monkeypatch.setattr(os.path, 'isdir', lambda x: True)
        monkeypatch.setattr(config, 'git_cache_dir', '/path/to')

        repo = 'http://pkgs.fedoraproject.org/git/rpms-checks/firefox.git'
        tasks = utils.discover_tasks('koji_build', repo)
        import pprint
        pprint.pprint(mock_check_output.call_args)
        pprint.pprint(mock__should_schedule.call_args)
        assert mock_check_output.call_args[0][0][6] == repo
        assert mock_check_output.call_args[0][0][
            7] == os.path.join('/path/to', 'firefox')
        assert mock__should_schedule.call_args[0][0] == os.path.join('/path/to',
                                                                     'firefox',
                                                                     'rpmlint',
                                                                     'runtask.yml')
        assert mock__should_schedule.call_args[0][1] == 'koji_build'
        assert tasks == ['rpmlint']


class TestGetBranch(object):

    def test_nvrs(self):
        assert utils.get_distgit_branch('foo-1.2-3.fc20') == 'f20'
        assert utils.get_distgit_branch('foo-1.2-3.fc20.1') == 'f20'
        assert utils.get_distgit_branch('foo-1.2-3.fc22hashgit.fc20') == 'f20'
        assert utils.get_distgit_branch('foo-1.2-3.fc22hashgit.fc20.1') == 'f20'

    def test_unsupported_dist_tag(self):
        with pytest.raises(exc.TriggerMsgError):
            utils.get_distgit_branch('foo-1.2-3.el7')
